--[[
===========================================================
	Initialization
===========================================================
]]
local L = AceLibrary("AceLocale-2.2"):new("Coins", true)

Coins = AceLibrary("AceAddon-2.0"):new(	"AceConsole-2.0",
										"AceEvent-2.0",
										"AceHook-2.1",
										"AceDB-2.0",
										"DataMethods-1.0")

function Coins:DefineDB()
	Coins:RegisterDB("CoinsDB", "CoinsDBPC")
	Coins:RegisterDefaults("profile", {
		cashFormat		= "full",
		colorCash		= true,
		colorText		= true,
		displayLoot		= true,
		displayAH		= false,
		displayMail		= true,
		displayMcht		= false,
		summaryAH		= true,
		summaryMail		= true,
		summaryMcht		= true
	} )
end

function Coins:DefineCmdOptions()
	self.cmdOpts = {
		type='group',
		args = {
			cashfmt		= {
				type	= "group",
				name	= L["Opt_CashFormatName"],
				desc	= L["Opt_CashFormatDesc"],
				args	= {
					cond	= {
						type	= 'toggle',
						name	= L["Opt_CashFmtCondName"],
						desc	= L["Opt_CashFmtCondDesc"],
						get		= function() return self:Is_cashFormat("cond") end,
						set		= function() self:Set_cashFormat("cond") end
					},
					ext		= {
						type	= 'toggle',
						name	= L["Opt_CashFmtExtName"],
						desc	= L["Opt_CashFmtExtDesc"],
						get		= function() return self:Is_cashFormat("ext") end,
						set		= function() self:Set_cashFormat("ext") end
					},
					full	= {
						type	= 'toggle',
						name	= L["Opt_CashFmtFullName"],
						desc	= L["Opt_CashFmtFullDesc"],
						get		= function() return self:Is_cashFormat("full") end,
						set		= function() self:Set_cashFormat("full") end
					},
					short	= {
						type	= 'toggle',
						name	= L["Opt_CashFmtShortName"],
						desc	= L["Opt_CashFmtShortDesc"],
						get		= function() return self:Is_cashFormat("short") end,
						set		= function() self:Set_cashFormat("short") end
					}
				}
			},
			colorcash	= {
				type	= 'toggle',
				name	= L["Opt_ColorCashName"],
				desc	= L["Opt_ColorCashDesc"],
				get		= "Is_colorCash",
				set		= "Toggle_colorCash",
			},
			colortext	= {
				type	= 'toggle',
				name	= L["Opt_ColorTextName"],
				desc	= L["Opt_ColorTextDesc"],
				get		= "Is_colorText",
				set		= "Toggle_colorText",
			},
			disploot	= {
				type	= 'toggle',
				name	= L["Opt_DisplayLootName"],
				desc	= L["Opt_DisplayLootDesc"],
				get		= "Is_displayLoot",
				set		= "Toggle_displayLoot",
			},
			dispah		= {
				type	= 'toggle',
				name	= L["Opt_DisplayAHName"],
				desc	= L["Opt_DisplayAHDesc"],
				get		= "Is_displayAH",
				set		= "Toggle_displayAH",
			},
			dispmail	= {
				type	= 'toggle',
				name	= L["Opt_DisplayMailName"],
				desc	= L["Opt_DisplayMailDesc"],
				get		= "Is_displayMail",
				set		= "Toggle_displayMail",
			},
			dispmcht	= {
				type	= 'toggle',
				name	= L["Opt_DisplayMchtName"],
				desc	= L["Opt_DisplayMchtDesc"],
				get		= "Is_displayMcht",
				set		= "Toggle_displayMcht",
			},
			sumah		= {
				type	= 'toggle',
				name	= L["Opt_SummaryAHName"],
				desc	= L["Opt_SummaryAHDesc"],
				get		= "Is_summaryAH",
				set		= "Toggle_summaryAH",
			},
			summail		= {
				type	= 'toggle',
				name	= L["Opt_SummaryMailName"],
				desc	= L["Opt_SummaryMailDesc"],
				get		= "Is_summaryMail",
				set		= "Toggle_summaryMail",
			},
			summcht		= {
				type	= 'toggle',
				name	= L["Opt_SummaryMchtName"],
				desc	= L["Opt_SummaryMchtDesc"],
				get		= "Is_summaryMcht",
				set		= "Toggle_summaryMcht",
			},
		}
	}

	self:RegisterChatCommand({L["SlashCmd1"], L["SlashCmd2"]}, self.cmdOpts)
end

function Coins:OnInitialize()
	-- Grab the loot split prefix so we can use it later to suppress the default
	-- money split message and avoid a double money gain message.
	self.displayOpt = "displayLoot"
	self.lootSplitPrefix = gsub(LOOT_MONEY_SPLIT, "%%s.*", "")
	self.abacus = AceLibrary("Abacus-2.0")
	self:CreateDataMethods()
	self:DefineDB()
	self:DefineCmdOptions()
	self.cashFmtRef = {
		cond	= "FormatMoneyCondensed",
		ext		= "FormatMoneyExtended",
		full	= "FormatMoneyFull",
		short	= "FormatMoneyShort",
	}
end

function Coins:OnEnable()
	-- Save initial starting cash
	self.cash = GetMoney()

	self:RegisterEvent("PLAYER_MONEY", "DisplayMoneyChange")
	self:RegisterEvent("AUCTION_HOUSE_SHOW", "OpenTransactionsAH")
	self:RegisterEvent("AUCTION_HOUSE_CLOSED", "CloseTransactions")
	self:RegisterEvent("MAIL_SHOW", "OpenTransactionsMail")
	self:RegisterEvent("MAIL_CLOSED", "CloseTransactions")
	self:RegisterEvent("MERCHANT_SHOW", "OpenTransactionsMcht")
	self:RegisterEvent("MERCHANT_CLOSED", "CloseTransactions")
	self:Hook("ChatFrame_OnEvent", "IgnoreGameMessages", true)
end

function Coins:Msg(text)
	DEFAULT_CHAT_FRAME:AddMessage(text)
end


--[[
===========================================================
	Data Management
===========================================================
]]
function Coins:CreateDataMethods()
	self:CreateCompareMethods("cashFormat")
	self:CreateToggleMethods("colorCash")
	self:CreateToggleMethods("colorText")
	self:CreateToggleMethods("displayLoot")
	self:CreateToggleMethods("displayAH")
	self:CreateToggleMethods("displayMail")
	self:CreateToggleMethods("displayMcht")
	self:CreateToggleMethods("summaryAH")
	self:CreateToggleMethods("summaryMail")
	self:CreateToggleMethods("summaryMcht")
end


--[[
===========================================================
	Cash Display
===========================================================
]]
function Coins:DisplayMoneyChange()
	local cashNow = GetMoney()
	local func		= self.cashFmtRef[self.db.profile.cashFormat]
	local cashStr	= self.abacus[func](self.abacus,
										abs(cashNow - self.cash),
										self:Is_colorCash(),
										self:Is_colorText())

	if self.displayOpt and self.db.profile[self.displayOpt] then
		-- Have to have both checks for > or < because the money change event will actually
		-- fire sometimes when there is no change.
		if cashNow > self.cash then
			self:Msg(format(L["CashGainMsg"], cashStr))
		elseif cashNow < self.cash then
			self:Msg(format(L["CashLossMsg"], cashStr))
		end
	end

	self.cash = cashNow
end

function Coins:DisplaySummary()
	local func		= self.cashFmtRef[self.db.profile.cashFormat]
	local cashStr	= self.abacus[func](self.abacus,
										abs(self.cashNow - self.cashOnOpen),
										self:Is_colorCash(),
										self:Is_colorText())

	if self.cashNow > self.cashOnOpen then
		self:Msg(format(L["CashGainSumMsg"], cashStr))
	elseif self.cashNow < self.cashOnOpen then
		self:Msg(format(L["CashLossSumMsg"], cashStr))
	end
end

function Coins:OpenTransactionsAH()
	self.displayOpt = "displayAH"
	if self.db.profile.summaryAH then self.cashOnOpen = GetMoney() end
end

function Coins:OpenTransactionsMail()
	self.displayOpt = "displayMail"
	if self.db.profile.summaryMail then self.cashOnOpen = GetMoney() end
end

function Coins:OpenTransactionsMcht()
	self.displayOpt = "displayMcht"
	if self.db.profile.summaryMcht then self.cashOnOpen = GetMoney() end
end

function Coins:CloseTransactions()
	self.displayOpt = "displayLoot"
	self.cashNow	= GetMoney()
	if self.cashOnOpen and (self.cashOnOpen ~= self.cashNow) then self:DisplaySummary() end
	self.cashOnOpen = nil
end

function Coins:IgnoreGameMessages(event)
	if arg1 and strfind(arg1, self.lootSplitPrefix) then return end
	self.hooks["ChatFrame_OnEvent"](event)
end
