--[[
Name: DataMethods-1.0
Revision: $Rev: 0 $
Author(s): Turan (turan@gryffon.com)
Website: http://www.wowace.com
Documentation:
SVN: http://svn.wowace.com/root/trunk/DataMethodsLib/DataMethods-1.0
Description: A mixin to create routine methods for data management.
Dependencies: AceLibrary, AceOO
]]

local MAJOR_VERSION = "DataMethods-1.0"
local MINOR_VERSION = "$Revision: 0 $"

if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end
if not AceLibrary:HasInstance("AceOO-2.0") then error(MAJOR_VERSION .. " requires AceOO-2.0") end

local AceOO = AceLibrary("AceOO-2.0")
local Mixin = AceOO.Mixin
local Lib	= Mixin {
						"CreateGetMethod",
						"CreateCompareMethod",
						"CreateSetMethod",
						"CreateToggleMethods",
						"CreateCompareMethods"
					}

function Lib:CreateGetMethod(field, prefix)
	self[(prefix or "Get_")..field] = function(self) return self.db.profile[field] end
end

function Lib:CreateCompareMethod(field, prefix)
	self[(prefix or "Is_")..field] = function(self, value) return self.db.profile[field] == value end
end

function Lib:CreateSetMethod(field, prefix)
	self[(prefix or "Set_")..field] = function(self, value) self.db.profile[field] = value end
end

function Lib:CreateToggleMethods(field)
	self:CreateGetMethod(field, "Is_")
	self:CreateSetMethod(field, "Toggle_")
end

function Lib:CreateCompareMethods(field)
	self:CreateCompareMethod(field)
	self:CreateSetMethod(field)
end


AceLibrary:Register(Lib, MAJOR_VERSION, MINOR_VERSION)
Lib = nil
